import logging
import threading
import time

import redis

gitem = None


def skip_frame(callback):
    global gitem
    while 1:
        if callback and gitem is not None:
            callback(gitem['data'], 'robot_reids_sub', '', time.time())


class Subscriber(object):
    _instance_lock = threading.Lock()

    def __init__(self,
                 name='robot_reids_sub',
                 host='127.0.0.1',
                 port='6379',
                 db=3,
                 health_check_interval=1):
        self.name = name
        self.host, self.port = host, port
        self.db, self.health_check_interval = db, health_check_interval
        self.pool = redis.ConnectionPool(host=host, port=port, db=self.db)
        logging.debug('create a redis pool at {}:{} for subscriber {}'.format(
            host, port, name))

    def __new__(cls, *args, **kwargs):
        if not hasattr(Subscriber, '_instance'):
            with Subscriber._instance_lock:
                if not hasattr(Subscriber, '_instance'):
                    Subscriber._instance = object.__new__(cls)
        return Subscriber._instance

    def get_connection(self):
        return redis.StrictRedis(connection_pool=self.pool,
                                 db=self.db,
                                 socket_connect_timeout=4,
                                 retry_on_timeout=True,
                                 socket_timeout=4)

    def recv(self, topic, callback=None):
        global gitem
        t = threading.Thread(target=skip_frame, args=[
            callback,
        ])
        t.start()
        rc = Subscriber._instance.get_connection()
        name = Subscriber._instance.name
        ps = rc.pubsub()
        ps.subscribe(topic)
        try:
            for item in ps.listen():
                gitem = item
                logging.debug(
                    'Redis server {}\'s top {} subscribe a data at {}.'.format(
                        name, topic, time.time()))
        except redis.exceptions.ConnectionError:
            logging.error('connect close by server, retry...')
            host, port = \
                Subscriber._instance.host, Subscriber._instance.port
            Subscriber._instance.rc = Subscriber._instance.get_connection()
            logging.error('reconnect at {}:{}'.format(host, port))
            self.recv(topic, callback=callback)



import logging
import threading
import time

import redis


class Publisher(object):
    _instance_lock = threading.Lock()

    def __init__(self,
                 name='robot_reids_pub',
                 host='127.0.0.1',
                 port='6379',
                 db=3,
                 health_check_interval=1):
        self.name = name
        self.db = db
        self.pool = redis.ConnectionPool(host=host, port=port)
        logging.debug('create a redis pool at {}:{} for publisher {}'.format(
            host, port, name))

    def __new__(cls, *args, **kwargs):
        if not hasattr(Publisher, '_instance'):
            with Publisher._instance_lock:
                if not hasattr(Publisher, '_instance'):
                    Publisher._instance = object.__new__(cls)
        return Publisher._instance

    def get_connection(self):
        return redis.StrictRedis(connection_pool=self.pool,
                                 db=self.db,
                                 socket_connect_timeout=4,
                                 retry_on_timeout=True,
                                 socket_timeout=4)

    def send(self, topic, dpack):
        rc = Publisher._instance.get_connection()
        name = Publisher._instance.name
        rc.publish(topic, dpack)

        logging.debug('Redis server {}\'s top {} publish a data at {}.'.format(
            name, topic, time.time()))



from pubsub import configs
import unittest
import os
import sys
import logging


class ConfigsTest(unittest.TestCase):
    def setUp(self):
        # use ROBOT_CONFIGS
        _root = sys.path[0]
        logging.debug('run in root: {}'.format(_root))
        os.environ.setdefault(configs.ROBOT_CONFIGS_PATH,os.path.join(_root, 'config')),
        _configs = configs.Configs()
        self.configs = _configs
        self.root = _root

    def testFileNotExist(self):
        _file = os.path.join(self.root, 'config', 'notexist.yaml')
        try:
            config = configs.Configs(_file)
            self.assertTrue(False)
        except Exception as e:
            self.assertEqual(e.message, configs.INVALID_CONFIG.format(_file))
    
    def testEmptyFile(self):
        _file = os.path.join(self.root, 'config', 'empty.yaml')
        try:
            _configs = configs.Configs(_file)
            self.assertEqual('a empty file', _file)
        except Exception as e:
            self.assertEqual(e.message, configs.CANNOT_LOAD.format(_file))

    def testCustomFile(self):
        _file = os.path.join(self.root, 'config', 'custom.yaml')
        try:
            _configs = configs.Configs(_file)
            # self.assertTrue(True)
        except Exception as e:
            self.assertTrue(False)
    
    def testInvalidModel(self):
        _model = 'nomodel'
        try:
            val = self.configs.get_config(_model)
        except Exception as e:
            self.assertEqual(e.message, configs.INVALID_MODEL.format(_model))

    def testModelisNone(self):
        _model = None
        try:
            val = self.configs.get_config(_model)
        except Exception as e:
            self.assertEqual(e.message, configs.INVALID_MODEL.format(_model))

    def testParamisNone(self):
        _model = 'test'
        _param = None
        # test:
        #     type: "test"
        val = self.configs.get_config(_model, _param)
        self.assertDictEqual(val, {'type': 'test'})

    def testInvalidParam(self):
        _model = 'test'
        _param = 'None'
        # test:
        #     type: "test"
        try:
            val = self.configs.get_config(_model, _param)
            self.assertDictEqual(val, {'type': 'test'})
        except Exception as e:
            self.assertEqual(e.message, configs.INVALID_PARAM.format(_model, _model, _param))



import logging
import struct
import time

import cv2
import numpy as np

from pubsub.publisher import Publisher
from pubsub.subscriber import Subscriber


def packImage(img, timestamp=None):
    h, w = img.shape[:2]
    shape = struct.pack('>II', h, w)
    img_time = struct.pack('>d', timestamp or time.time())
    encoded = shape + img_time + img.tobytes()
    return encoded


def unpackImage(encoded):
    h, w = struct.unpack('>II', encoded[:8])
    img_time = struct.unpack('>d', encoded[8:16])
    img = np.frombuffer(encoded, dtype=np.uint8, offset=16).reshape(h, w, 3)
    return img, img_time[0]


class CameraPub(object):
    def __init__(self,
                 pipe=0,
                 name='robot_mono',
                 host='127.0.0.1',
                 port='6379',
                 db=3):
        self.cap = cv2.VideoCapture(pipe)  # video capture object
        self.cap.set(cv2.CAP_PROP_BUFFERSIZE, 3)  # set buffer size
        self.pub = Publisher(name='{}_pub'.format(name),
                             host=host,
                             port=port,
                             db=db)

    def publish(self, topic='robot_mono', flip=2):
        ret_val, img = self.cap.read()
        timestamp = time.time()
        while ret_val:
            pdata = packImage(img, timestamp)
            self.pub.send(topic, pdata)
            ret_val, img = self.cap.read()
            timestamp = time.time()


class CameraSub(object):
    def __init__(self,
                 pipe=0,
                 name='robot_mono',
                 host='127.0.0.1',
                 port='6379',
                 db=3):
        self.sub = Subscriber(name='{}_sub'.format(name),
                              host=host,
                              port=port,
                              db=db)

    def subscribe(self, topic='robot_mono', callback=None):
        self.sub.recv(topic, callback or self.process)

    def process(self, pdata, *arg):
        if 1 == pdata:
            logging.debug('not a stable connection, pass that message')
        else:
            img, _ = unpackImage(pdata)
            # do what you want in your own process
            cv2.imshow('process', img)
            if cv2.waitKey(1) == ord('q'):
                cv2.destroyAllWindows()



import logging
import os

import yaml

INVALID_CONFIG = 'invalid config file: {}'
CANNOT_LOAD = 'can not load config fle: {}'
CUSTOM_CONFIG = 'use custom config file: {}'
SYSTEM_CONFIG = 'use system config file: {}'
INVALID_MODEL = 'invalid config model or model is not configured: {}'
INVALID_PARAM = 'invalid param in config model {} or param is not configured in config model {}: {}'

ROBOT_CONFIGS_PATH = 'ROBOT_CONFIGS'
CONFIG_FILES = 'pubsub.yaml'


class Configs():
    def __init__(self, config_file=None):
        if config_file is None:
            logging.debug(SYSTEM_CONFIG.format(config_file))
            config_file = os.path.join(os.getenv(ROBOT_CONFIGS_PATH),
                                       CONFIG_FILES)
        else:
            logging.debug(CUSTOM_CONFIG.format(config_file))

        self.config_file = config_file
        valid, msg = self.check_config_file()
        if not valid:
            raise Exception(msg)
        configs = self.get_configs()
        if configs is None:
            msg = CANNOT_LOAD.format(self.config_file)
            logging.error(msg)
            raise Exception(msg)
        self.configs = configs

    def check_config_file(self):
        msg = ''
        if not os.path.isfile(self.config_file):
            msg = INVALID_CONFIG.format(self.config_file)
            logging.error(msg)
            return False, msg
        return True, msg

    def get_configs(self):
        config = None
        if os.path.isfile(self.config_file):
            with open(self.config_file, 'r', encoding='utf-8') as f:
                config = yaml.safe_load(f.read(), )
        return config

    def get_config(self, model, param=''):
        res = None
        if model is None or model.strip() == '':
            msg = INVALID_MODEL.format(model)
            logging.error(msg)
            raise Exception(msg)
        try:
            config = self.configs[model]
        except KeyError as e:
            msg = INVALID_MODEL.format(model)
            logging.error(msg)
            logging.error(e.message)
            raise Exception(INVALID_MODEL.format(model))
        if param is None or param.strip() == '':
            res = config
        else:
            try:
                res = config[param]
            except KeyError as e:
                msg = INVALID_PARAM.format(model, model, param)
                logging.error(msg)
                logging.error(e.message)
                raise Exception(INVALID_PARAM.format(model, model, param))

        return res



import os
import re
import sys
from distutils.command.register import register as register_orig
from distutils.command.upload import upload as upload_orig

from setuptools import find_packages, setup

if sys.version_info < (3, 0):
    from io import open

try:
    # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:
    # for pip <= 9.0.3
    from pip.req import parse_requirements

project_path = os.path.dirname(__file__)


class register(register_orig):
    def _get_rc_file(self):
        return os.path.join('.', '.pypirc')


class upload(upload_orig):
    def _get_rc_file(self):
        return os.path.join('.', '.pypirc')


def load_requirements():
    fname = os.path.join(project_path, 'requirements.txt')
    reqs = parse_requirements(fname, session=False)
    requirements = []
    try:
        requirements = [str(ir.req) for ir in reqs]
    except Exception as e:
        print(e)
        requirements = [str(ir.requirement) for ir in reqs]
    return requirements


def load_description():
    fname = os.path.join(project_path, 'README.md')
    with open(fname, encoding='utf-8') as f:
        return f.read()


def load_version(*file_paths):
    """This pattern was modeled on a method from the Python Packaging User
    Guide:

    https://packaging.python.org/en/latest/single_source_version.html
    We read instead of importing so we don't get import errors if our code
    imports from dependencies listed in install_requires.
    """
    base_module_file = os.path.join(*file_paths)
    with open(base_module_file) as f:
        base_module_data = f.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              base_module_data, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


# def load_version():
#     fname = os.path.join(project_path, 'pubsub/version.py')
#     with open(fname, 'r', encoding='utf-8') as f:
#         exec(compile(f.read(), fname, 'exec'))
#     return locals()['__version__']

setup(
    name='aa-pubsub',
    version=load_version(project_path, 'pubsub/version.py'),
    #   py_modules=['pubsub.publisher', 'pubsub.subscriber', 'pubsub.camera'],
    author='shuyuanhao',
    author_email='shuyuanhao@cetiti.com',
    keywords='redis publisher&subscriber',
    url='https://git.d.com/hik-aa-robots/robots-intelligent-systems/pubsub',
    description='a publish&subscrib toolkit',
    packages=find_packages(),
    install_requires=load_requirements(),
    entry_points={'console_scripts': ['camera_redis=app:main']},
    long_description=load_description(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha', 'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License (GPL)'
    ],
    zip_safe=False,
    cmdclass={
        'register': register,
        'upload': upload,
    })
